using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Palisade.RdkPia;


namespace SimpleSimCSharp
{
	/// <summary>
	/// This examples shows how to:
    /// * define discrete distribution
    /// * define general distribution
    /// * get results of sensitivity analysis
	/// </summary>


	public class Form1 : System.Windows.Forms.Form
	{
		private RDKApplication MyRDKApp;

        private RDKOutput m_Output;
        private RDKInput m_Input1;
        private RDKInput m_Input2;

        private System.Windows.Forms.Button button1;
        private TextBox textBox1;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			
			MyRDKApp = new RDKApplicationClass();
			bool UseInterface;
			UseInterface = false;
			MyRDKApp.Initialize(ref UseInterface);

			string path = "";
            if (UseInterface)
            {
			    RDKInterfaceBootMode mode = RDKInterfaceBootMode.RDKBootMinimized;
			    MyRDKApp.Interface.Start(ref path, ref mode);
            }

			// Declare which procedures handle which events.
			MyRDKApp.Iteration += new __RDKApplication_IterationEventHandler(MyIterationEventHandler);
			MyRDKApp.SimShouldStart += new __RDKApplication_SimShouldStartEventHandler(MySimShouldStartEventHandler);
			button1.Click += new System.EventHandler(btn_Click);

		}



		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			MyRDKApp.Free();
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.button1 = new System.Windows.Forms.Button();
            this.textBox1 = new System.Windows.Forms.TextBox();
            this.SuspendLayout();
            // 
            // button1
            // 
            this.button1.Location = new System.Drawing.Point(69, 114);
            this.button1.Name = "button1";
            this.button1.Size = new System.Drawing.Size(133, 48);
            this.button1.TabIndex = 0;
            this.button1.Text = "Start Simulation";
            // 
            // textBox1
            // 
            this.textBox1.Location = new System.Drawing.Point(44, 21);
            this.textBox1.Multiline = true;
            this.textBox1.Name = "textBox1";
            this.textBox1.Size = new System.Drawing.Size(209, 65);
            this.textBox1.TabIndex = 2;
            this.textBox1.Text = "This example shows how to define Discrete and General distribuitons, and how to g" +
                "et sensitivity analysis results, using RDK 4.1 and C#.";
            // 
            // Form1
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(292, 197);
            this.Controls.Add(this.textBox1);
            this.Controls.Add(this.button1);
            this.Name = "Form1";
            this.Text = "Form1";
            this.ResumeLayout(false);
            this.PerformLayout();

		}
		#endregion


		private void btn_Click(object sender, System.EventArgs e)
		{
			RunSimulation();
			
			// To get results in the form of arrays,
			// we first need to set ArrayBaseOption to 0
			// since in .Net arrays are (generally) zero-based
			short ArrayBase = 0;
			MyRDKApp.set_ArrayBaseOption(ref ArrayBase);

			// Get regression coefficients.
            int WhichSim = 1;
            RDKRegressionData[] regrData = null;
            object[] objArray = new object[0];
            object myObj = objArray;
            m_Output.get_Results(ref WhichSim).Regression(ref regrData, ref myObj);

            MessageBox.Show("Regression coefficients: " + regrData[0].StandardBCoeff.ToString("0.00") + " " + regrData[1].StandardBCoeff.ToString("0.00"));

	}

		private void RunSimulation()
		{
			

			// Clear any existing run settings
			MyRDKApp.ClearModel();

			// Add output
			string OutputName = "Output";
			m_Output = MyRDKApp.Outputs.Add(ref OutputName);


            // Add discrete input
            RDKDistribution l_Distribution = new RDKDistributionClass();
            RDKDistributionType l_Type = RDKDistributionType.RDKDistTypeDISCRETE;
            l_Distribution.set_DistType(ref l_Type);
            double[] xValues = new double[2];
            double[] pValues = new double[2];
            xValues[0] = 10;
            xValues[1] = 20;
            pValues[0] = 1;
            pValues[1] = 1;
            short xORp = 0;
            l_Distribution.SetValueTable(ref xORp, ref xValues);
            xORp = 1;
            l_Distribution.SetValueTable(ref xORp, ref pValues);

            object l_DistrObj = l_Distribution;
			string l_InputName = "Input1";
            m_Input1 = MyRDKApp.Inputs.Add(ref l_InputName, ref l_DistrObj);

            //  Add general input
            l_Distribution = new RDKDistributionClass();
            l_Type = RDKDistributionType.RDKDistTypeGENERAL;
            l_Distribution.set_DistType(ref l_Type);
            double min = 100;
            double max = 130;
            short i = 1;
            l_Distribution.set_Arg(ref i, ref min);
            i = 2;
            l_Distribution.set_Arg(ref i, ref max);
            xValues = new double[2];
            pValues = new double[2];
            xValues[0] = 110;
            xValues[1] = 120;
            pValues[0] = 1;
            pValues[1] = 1;
            xORp = 0;
            l_Distribution.SetValueTable(ref xORp, ref xValues);
            xORp = 1;
            l_Distribution.SetValueTable(ref xORp, ref pValues);

            l_DistrObj = l_Distribution;
            l_InputName = "Input2";
            m_Input2 = MyRDKApp.Inputs.Add(ref l_InputName, ref l_DistrObj);
			

			// run simulation
			int NumOfIterations = 2000;
			MyRDKApp.SimSettings.set_NumIterations(ref NumOfIterations);

			MyRDKApp.Simulate();

		}

	
		private void MyIterationEventHandler(ref int whichSim, ref int whichIter, ref bool cancel)
		{
            // output is the sum of inputs
            double x = 0;
            x = m_Input1.Value + m_Input2.Value;
            m_Output.set_Value(ref x);

		}
		private void MySimShouldStartEventHandler()
		{
			RunSimulation();
		}
		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());

		}


	}
}
